<?php

namespace WebServiceCall\Repository\WS\Data;

abstract class ASoapParams {
    private const PARAMS_DEFAULT_USAGE_MSG = "Estes dados são utilizadas para realizar uma buscar no Web Service da Conciliadora via SOAP.";
    private const EMPRESA_PARAMS_NOT_FOUND = "As credenciais da empresa não foram informadas.";
    private const PERIOD_NOT_FOUND = "O período das vendas não foi informado.";
    
    private $idEmpresa;
    private $password;
    private $dtIni;
    private $dtFim;

    public function __construct($idEmpresa, $password, $dtIni, $dtFim) {
        $this->idEmpresa = $idEmpresa;
        $this->password = $password;
        $this->dtIni = $dtIni;
        $this->dtFim = $dtFim;
    }
    
    public function getIdEmpresa() {
        return $this->idEmpresa;
    }
    
    public function setIdEmpresa($idEmpresa) {
        $this->idEmpresa = $idEmpresa;
    }
    
    public function getPassword() {
        return $this->password;
    }
    
    public function setPassword($password) {
        $this->password = $password;
    }
    
    public function getDtIni() {
        return $this->dtIni;
    }
    
    public function setDtIni($dtIni) {
        $this->dtIni = $dtIni;
    }
    
    public function getDtFim() {
        return $this->dtFim;
    }
    
    public function setDtFim($dtFim) {
        $this->dtFim = $dtFim;
    }

    public function getCallParams() {
        $this->validateEmpresaParams();
        $this->validatePeriod();
        return $this->toArray();
    }
    
    private function validateEmpresaParams() {
        if( empty($this->idEmpresa) || 
            empty($this->password)) {
            throw new \Exception(self::EMPRESA_PARAMS_NOT_FOUND . ' ' . self::PARAMS_DEFAULT_USAGE_MSG);
        }
    }

    private function validatePeriod() {
        if( empty($this->dtIni) || 
            empty($this->dtFim)) {
            throw new \Exception(self::PERIOD_NOT_FOUND .  ' ' . self::PARAMS_DEFAULT_USAGE_MSG);
        }
    }
    
    public abstract function toArray();
    
    public abstract function getXmlXPath();
    
    public abstract function getServiceName();
    
    public abstract function getURL();
    
    public abstract function getResponseData($response);
    
    public abstract function getReportID();
}

