<?php

namespace WebServiceCall\Repository\WS;

use WebServiceCall\Log\OutputStreamLogger;

use WebServiceCall\Log\IWebServiceLogger;
use WebServiceCall\Log\WebServiceLogger;

use WebServiceCall\SoapClient\ISoapHandler;
use WebServiceCall\SoapClient\SoapHandler;

use WebServiceCall\Repository\WS\Data\ASoapParams;

class WebServiceHandler implements IWebServiceHandler {
    private const NO_SESSION_ID = 'Você deve buscar os dados do Web Service da Conciliadora, antes de buscar o ID dessa sessão.';
    
    private $logger;

    private $nrSeqControle;

    private $accessDate;
    private $responseDate;

    public function __construct () {
        $this->setWebServiceLog(new WebServiceLogger ());         
        $this->nrSeqControle = NULL;
    }
    
    public function getAccessDate() {
        return $this->accessDate;
    }
    
    public function setAccessDate() {
        $this->accessDate = date('d/m/Y H:i:s');
    }
    
    public function getResponseDate() {
        return $this->responseDate;
    }
    
    public function setResponseDate() {
        $this->responseDate = date('d/m/Y H:i:s');
    }

    private function setWebServiceLog(IWebServiceLogger $logService) {
        $this->logger = $logService;
    }
    
    private function recursiveXMLUnify($xml, $xmlXPath) {
        $wsDataArray = [];
        $uniqueSale = [];
        $data = $xml->xpath($xmlXPath);
        foreach($data as $d) {
            foreach($d as $key => $value) {
                $uniqueSale[':' . (string) $key] = (string) $value;
            }
            $wsDataArray[] = $uniqueSale;
            $uniqueSale = [];
        }
        
        return $wsDataArray;
    }
    
    private function setAccessID($accessID) {
        $this->nrSeqControle = $accessID;
    }
    
    // @Override
    public function getAccessID() {
        if(!$this->nrSeqControle) {
            throw new \Exception(self::NO_SESSION_ID);
        }
        
        return $this->nrSeqControle;
    }
    
    private function insertAccessControl($reportID) {
        $nextID = $this->logger->insertAccessControl($reportID, $this->getAccessDate(), $this->getResponseDate());
        $this->setAccessID($nextID);
        return $nextID;
    }
    
    // @Override
    // Atualiza a tabela de controle de acesso para saber qual o estabelecimento o WS está tratando.
    public function updateAccessControl($nrEstabelec) {
        $this->logger->updateAccessControl($nrEstabelec, $this->nrSeqControle);
    }
    
    // @Override
    public function getWebServiceData(ASoapParams $soapParams) {
        // Cria a conexão via Soap
        $soapConn = new SoapHandler($soapParams->getURL(), $soapParams->getServiceName());
        
        // Salva a data de tentativa de acesso ao WebService.
        $this->setAccessDate();

        // Inicia a conexão com o serviço SOAP.
        $response = $soapConn->soapCall($soapParams->getCallParams());

        // Define o date em que a resposta do WS foi recebida
        $this->setResponseDate();
        
        // Lê os dados do WebService.
        // Link para a documentação da classe SimpleXMLElement - http://php.net/manual/en/class.simplexmlelement.php
        $data = $this->recursiveXMLUnify(
                    new \SimpleXMLElement($soapParams->getResponseData($response) ),
                    $soapParams->getXmlXPath()
        );
        
        if (count($data) === 0) {
            $msg = PHP_EOL . "Não houve retorno do serviço: " . $soapParams->getServiceName() . 
                                    " para a filial " . $soapParams->getIdEmpresa() . 
                                    " neste dia." . PHP_EOL . "Confira as credenciais da empresa desejada!";
            throw new \Exception($msg);
        }


        // Salva no banco o último acesso ao WebService de Vendas.
        // Persistir o acesso para o relatório de vendas.
        $this->nrSeqControle = $this->insertAccessControl($soapParams->getReportID());
        
        return $data;
    }

}
